% PStill run-time script, F. Siegert, Oktober 1998, v1.02
% Use this to do specific additions to the PDF engine
% DO NOT TOUCH IT IF YOU DO NOT UNDERSTAND IT EXCEPT FOR THE LINE
% MARKED **** IN CASE OF TROUBLES WITH ANNOTATIONS

% Define variables
variable annotstring = "";
variable viewstring = "";
variable validAnnotation = 0;
variable insertionPage = 0;

% Utility function to produce output for the log file
define printf (n)
{
   fputs (Sprintf (n), stdout); pop ();
   fflush (stdout); pop ();
}

% *** Further interface functions are

% void objToAnnotation(int pagenumber, string annotation)
%      adds an annotation to the page, annotation must be a valid PDF structure
%      (see PDFSPEC.pdf from Adobe, Inc.), to find out the current page number
%      use curPageNum()

% int curPageNum()
%      returns the current page number

% addToContent(string line)
%      adds arbitary PDF code to the page contents

% addToPageObj(string key, string value)
%      adds a key/value pair to the current page description
%      (useful to set /rotate and friends, depending on the page number)

% *** Actions are called by PDF engine whenever a special event happens

% action_pdfmark - collects and filters PDF marks (annotations)
% called whenever a pdfmark dicionary entity is worked on
% s is a string and empty on end of dictionary
% n is a integer counter

define action_pdfmark(s,n) {

	% the string will be zero length at the end of the annoation, so process the collected data
	if (strlen(s) == 0) {
		if (validAnnotation > 0) {
			% We need to construct a destination for the link
			% #PAGEANNOT=### is a placeholder and will be replaced by dumpPDFBase() by the
			% object number for the ### page object at dumpTime, we take the rest from /View
			variable destination;
			if (strlen(viewstring) > 0) {
				destination = Sprintf("\n/Dest [ #PAGEANNOT=%d %s", insertionPage, viewstring ,2 );
			} else {
				destination = Sprintf("\n/Dest [ #PAGEANNOT=%d ]", insertionPage ,1 );
			}
			annotstring = strcat(annotstring,destination);

			% Define the complete annotation 
			% **** UNCOMMENT NEXT LINE TO TURN OFF PDF ANNOTATION PROCESSING
			objToAnnotation(curPageNum(),annotstring );
		}
		% set initial state
		annotstring = "";
		viewstring = "";
		validAnnotation = 0;
		return;
	}

	% is it the /Page key, we need to remember the target page, then drop key
	if (strncmp(s,"/Page",5) == 0) {
		insertionPage = integer( substr(s, 6, -1) );
		return;
	}

	% is it the /View key, we need to remember the target location, then drop key
	if (strncmp(s,"/View",5) == 0) {
		viewstring = substr(s, 8, -1);
		return;
	}

	% be sure to set a newline before adding anything
	if (strlen(annotstring) > 0) {
		annotstring = strcat(annotstring,"\n");
	} else {
		% set the current page as default target page
		insertionPage = curPageNum();

		% we process only old style LNK at the moment
		if (strcmp(s,"/LNK") == 0) {
			validAnnotation++;
			s = "/Link";
		}
	}

	% add to annotation, then return for next key/value pair string
	annotstring = strcat(annotstring,s);
	
}

% action_pagestart - called on start of each page description
define action_pagestart() {
	% addToContent(" % Enter PDF code here\n");

	% this is a good place to insert new key/value pairs to the page description
	% a nice one is e.g. to rotate only page 3 by 180 degrees
	% if (curPageNum() == 3) addToPageObj("/Rotate","180");
}

% action_pageend - called on end of each page description
define action_pageend() {
	% addToContent(" % Enter PDF code here\n");
}

% END OF CONTROL SCRIPT